@@ -86,8 +86,12 @@ class PrivateKey:
         returns the key's raw bytes
     sign_message(message, compressed=True)
         signs the message's digest and returns the signature
-    sign_transaction(tx, compressed=True)
-        signs the transaction's digest and returns the signature
+    sign_input(tx, txin_index, script, sighash=SIGHASH_ALL)
+        signs the transaction's digest for a particular index and returns the
+        signature.
+    sign_segwit_input(tx, txin_index, script, amount, sighash=SIGHASH_ALL)
+        signs the transaction's digest for a particular index and amount and 
+        returns the signature.
     get_public_key()
         returns the corresponding PublicKey object
     """
@@ -694,21 +698,21 @@ class Address(ABC):
 
     @classmethod
     def from_address(cls, address):
-        """Creates and address object from an address string"""
+        """Creates an address object from an address string"""
 
         return cls(address=address)
 
 
     @classmethod
     def from_hash160(cls, hash160):
-        """Creates and address object from a hash160 string"""
+        """Creates an address object from a hash160 string"""
 
         return cls(hash160=hash160)
 
 
     @classmethod
     def from_script(cls, script):
-        """Creates and address object from a Script object"""
+        """Creates an address object from a Script object"""
 
         return cls(script=script)
 
@@ -948,21 +952,21 @@ class SegwitAddress(ABC):
 
     @classmethod
     def from_address(cls, address):
-        """Creates and address object from an address string"""
+        """Creates an address object from an address string"""
 
         return cls(address=address)
 
 
     @classmethod
     def from_hash(cls, witness_hash):
-        """Creates and address object from a hash string"""
+        """Creates an address object from a hash string"""
 
         return cls(witness_hash=witness_hash)
 
 
     @classmethod
     def from_script(cls, script):
-        """Creates and address object from a Script object"""
+        """Creates an address object from a Script object"""
 
         return cls(script=script)
 
